# run with expect -f v6install.tcl

set break "\005"
set eot "\004"
set timeout -1
set prompt ""
set user ""

proc sendline {s} {
	send -- $s
	send "\r"
}

proc simh {s} {
	expect "sim>"
	sendline $s
}

proc mboot {s} {
	expect "="
	sendline $s
}

proc uboot {s} {
	expect "@"
	sendline $s
}

proc unix {s} {
	expect "$::prompt "
	sendline $s
}

proc login {u} {
	global user
	global prompt
	set user $u
	expect "login: "
	sendline $user
	if { $user == "root" } {
		set prompt "#"
	} else {
		set prompt "%"
	}
	puts "prompt: $prompt"
	unix "stty cr0"
}

proc shutdown {} {
	# halt
	unix "sync"
	unix "sync"
	unix "sync"
	after 1000 { send $break }
}

proc install {} {
	uplevel #0 { spawn pdp11 install.ini }
	simh "g 100000"
	after 1000 { send $break }
	simh "g 0"

	# copy boot sector
	mboot "tmrk"
	expect "disk offset"
	sendline "0"
	expect "tape offset"
	sendline "100"
	expect "count"
	sendline "1"

	# copy root filesys
	mboot "tmrk"
	expect "disk offset"
	sendline "1"
	expect "tape offset"
	sendline "101"
	expect "count"
	sendline "3999"

	mboot $::break

	# now to boot UNIX
	simh "d sr 1"
	simh "boot rk0"
	uboot "rkunix"
	login "root"

	# make some devices
	unix "/etc/mknod /dev/rk0 b 0 0"
	unix "/etc/mknod /dev/rk1 b 0 1"
	unix "/etc/mknod /dev/rk2 b 0 2"
	unix "/etc/mknod /dev/rrk0 c 9 0"
	unix "/etc/mknod /dev/rrk1 c 9 1"
	unix "/etc/mknod /dev/rrk2 c 9 2"
	unix "/etc/mknod /dev/mt0 b 3 0"
	unix "/etc/mknod /dev/rmt0 c 12 0"

	# copy disks and mount them
	unix "dd if=/dev/mt0 of=/dev/rk1 count=4000 skip=4100"
	unix "dd if=/dev/mt0 of=/dev/rk2 count=4000 skip=8100"
	unix "/etc/mount /dev/rk1 /usr/source"
	unix "mkdir /usr/doc"
	unix "/etc/mount /dev/rk2 /usr/doc"

	# make them mount at boot
	unix "ed /etc/rc"
	sendline "\$a"
	sendline "/etc/mount /dev/rk1 /usr/source"
	sendline "/etc/mount /dev/rk2 /usr/doc"
	sendline "."
	sendline "w"
	sendline "q"

	shutdown

	simh "quit"
}

proc boot1 {} {
	uplevel #0 { spawn pdp11 boot.ini }
	uboot "rkunix"
	login "root"

	# build our kernel for an 11/40
	unix "rm hpunix rpunix unix"
	unix "chdir /usr/sys/conf"
	unix "cc mkconf.c"
	unix "mv a.out mkconf"
	unix "cat >myconf"
	sendline "rk"
	sendline "tm"
	sendline "pc"
	sendline "8dc"
	sendline "lp"
	send $::eot
	unix "mkconf <myconf"
	unix "as m40.s"
	unix "mv a.out m40.o"
	unix "cc -c c.c"
	unix "as l.s"
	unix "mv a.out l.o"
	unix "ld -x l.o m40.o c.o ../lib1 ../lib2"
	unix "mv a.out /unix"

	shutdown
	simh "quit"
}

proc eddisks {file r} {
	unix "ed ${file}.c"
	sendline "/${r}p/"
	sendline "d"
	sendline "-i"
	sendline "	\"/dev/${r}k0\","
	sendline "	\"/dev/${r}k1\","
	sendline "."
	sendline "w"
	sendline "q"
	unix "cc ${file}.c"
	unix "mv a.out /bin/$file"
}

proc boot2 {} {	
	uplevel #0 { spawn pdp11 boot.ini }
	uboot "unix"
	login "root"

	# create more device files
	unix "/etc/mknod /dev/lp0 c 2 0"
	unix "/etc/mknod /dev/pc c 1 0"
	unix "/etc/mknod /dev/tty0 c 3 0"
	unix "/etc/mknod /dev/tty1 c 3 1"
	unix "/etc/mknod /dev/tty2 c 3 2"
	unix "/etc/mknod /dev/tty3 c 3 3"
	unix "/etc/mknod /dev/tty4 c 3 4"
	unix "/etc/mknod /dev/tty5 c 3 5"
	unix "/etc/mknod /dev/tty6 c 3 6"
	unix "/etc/mknod /dev/tty7 c 3 7"

	# enable ttys
	unix "ed /etc/ttys"
	sendline "1,8s/./1/"
	sendline "w"
	sendline "q"

	# tell the system of our disks
	unix "chdir /usr/source/s1"
	eddisks df r
	eddisks icheck rr
	eddisks dcheck rr

	shutdown
	simh "quit"
}

install
boot1
boot2

expect eof
